@@ -223,6 +223,25 @@ void AsyncFizzClientT<SM>::tlsShutdown() {
   }
 }
 
+template <typename SM>
+void AsyncFizzClientT<SM>::shutdownWrite() {
+  DelayedDestruction::DestructorGuard dg(this);
+  // allow any previous writes as well as the close_notify to be written out
+  // before closing the socket
+  tlsShutdown();
+  transport_->shutdownWrite();
+}
+
+template <typename SM>
+void AsyncFizzClientT<SM>::shutdownWriteNow() {
+  DelayedDestruction::DestructorGuard dg(this);
+  // try to write out a close_notify although the ensuing shutdownWriteNow call
+  // on the underlying socket might prevent these bytes (and any other previous
+  // writes) from making it through
+  tlsShutdown();
+  transport_->shutdownWriteNow();
+}
+
 template <typename SM>
 void AsyncFizzClientT<SM>::closeWithReset() {
   DelayedDestruction::DestructorGuard dg(this);
@@ -608,16 +627,14 @@ void AsyncFizzClientT<SM>::ActionMoveVisitor::operator()(
 template <typename SM>
 void AsyncFizzClientT<SM>::ActionMoveVisitor::operator()(
     SecretAvailable& secret) {
-  client_.secretAvailable(secret.secret);
-#if 0
-  FOLLY_SDT(
-      fizz,
-      fizz_secret_available,
+  fizz_probe_secret_available(
       secret.secret.secret.size(),
       secret.secret.secret.data(),
-      secret.secret.type,
+      KeyLogWriter::secretToNSSLabel(secret.secret.type)
+          .value_or(std::numeric_limits<KeyLogWriter::Label>::max()),
       client_.getClientRandom()->data());
-#endif
+
+  client_.secretAvailable(secret.secret);
 }
 
 template <typename SM>
@@ -630,6 +647,11 @@ folly::Optional<CipherSuite> AsyncFizzClientT<SM>::getCipher() const {
   return getState().cipher();
 }
 
+template <typename SM>
+folly::Optional<NamedGroup> AsyncFizzClientT<SM>::getGroup() const {
+  return getState().group();
+}
+
 template <typename SM>
 std::vector<SignatureScheme> AsyncFizzClientT<SM>::getSupportedSigSchemes()
     const {
