@@ -15,8 +15,8 @@
 #include <fizz/client/FizzClientContext.h>
 #include <fizz/protocol/AsyncFizzBase.h>
 #include <fizz/protocol/Exporter.h>
+#include <fizz/util/Tracing.h>
 #include <folly/io/SocketOptionMap.h>
-#include <folly/tracing/StaticTracepoint.h>
 
 namespace fizz {
 namespace client {
@@ -111,6 +111,8 @@ class AsyncFizzClientT : public AsyncFizzBase,
   std::string getApplicationProtocol() const noexcept override;
 
   void tlsShutdown() override;
+  void shutdownWrite() override;
+  void shutdownWriteNow() override;
   void close() override;
   void closeWithReset() override;
   void closeNow() override;
@@ -135,6 +137,8 @@ class AsyncFizzClientT : public AsyncFizzBase,
 
   folly::Optional<CipherSuite> getCipher() const override;
 
+  folly::Optional<NamedGroup> getGroup() const override;
+
   std::vector<SignatureScheme> getSupportedSigSchemes() const override;
 
   Buf getExportedKeyingMaterial(
