@@ -20,22 +20,19 @@
 #include <system_error>
 
 namespace fizz {
-folly::AsyncSocket::ReadResult
-AsyncKTLSSocket::performRead(void** buf, size_t* buflen, size_t* /* offset */) {
+folly::AsyncSocket::ReadResult AsyncKTLSSocket::performReadMsg(
+    struct ::msghdr& msg,
+    AsyncReader::ReadCallback::ReadMode) {
 #if FIZZ_PLATFORM_CAPABLE_KTLS
-  struct iovec iov;
-  iov.iov_base = *buf;
-  iov.iov_len = *buflen;
+  // Reading of user ancillary data would have to be implemented, but
+  // currently conflicts with usage of `msg_control` below.
+  DCHECK(readAncillaryDataCallback_ == nullptr);
 
   // kTLS sends TLSInnerPlaintext.type in a 1 byte out-of-band cmsg payload.
   // The data that is read in `iov` is TLSInnerPlaintext.content
   char aux_data[CMSG_SPACE(1)];
 
-  struct msghdr msg {};
-  msg.msg_name = nullptr;
-  msg.msg_namelen = 0;
-  msg.msg_iov = &iov;
-  msg.msg_iovlen = 1;
+  // The caller already populated `msg.msg_iov*` and `msg.msg_name*`.
   msg.msg_control = aux_data;
   msg.msg_controllen = sizeof(aux_data);
   msg.msg_flags = MSG_DONTWAIT;
@@ -91,8 +88,10 @@ AsyncKTLSSocket::performRead(void** buf, size_t* buflen, size_t* /* offset */) {
     return ReadResult(bytes);
   }
 
-  return handleNonApplicationData(
-      content_type, folly::ByteRange(static_cast<unsigned char*>(*buf), bytes));
+  folly::IOBufQueue dataQ{folly::IOBufQueue::cacheChainLength()};
+  dataQ.append(folly::IOBuf::wrapIov(msg.msg_iov, msg.msg_iovlen));
+  dataQ.trimEnd(dataQ.chainLength() - bytes);
+  return handleNonApplicationData(content_type, std::move(dataQ));
 #else
   return ReadResult(
       READ_ERROR,
@@ -104,7 +103,7 @@ AsyncKTLSSocket::performRead(void** buf, size_t* buflen, size_t* /* offset */) {
 
 folly::AsyncSocket::ReadResult AsyncKTLSSocket::handleNonApplicationData(
     uint8_t type,
-    folly::ByteRange payload) {
+    folly::IOBufQueue payload) {
   // Ensure that if we have unparsed handshake data, then we are receiving
   // a continuation of handshake data. Alerts (e.g. close_notify) should not
   // interrupt a handshake message
@@ -118,7 +117,7 @@ folly::AsyncSocket::ReadResult AsyncKTLSSocket::handleNonApplicationData(
   }
 
   if (type == static_cast<uint8_t>(fizz::ContentType::alert)) {
-    return processAlert(payload);
+    return processAlert(std::move(payload));
   } else if (type == static_cast<uint8_t>(fizz::ContentType::handshake)) {
     // If we do not have a tls callback installed, then we cannot possibly
     // handle any post handshake messages.
@@ -129,10 +128,10 @@ folly::AsyncSocket::ReadResult AsyncKTLSSocket::handleNonApplicationData(
               folly::AsyncSocketException::SSL_ERROR,
               "kTLS received handshake data without a tls callback implementation to handle"));
     }
-    return processHandshakeData(payload);
+    return processHandshakeData(std::move(payload));
   } else {
     VLOG(7) << "Read a non data record, of type = " << (int)type << ": "
-            << folly::hexlify(payload);
+            << folly::hexlify(payload.move()->coalesce());
     return ReadResult(
         READ_ERROR,
         std::make_unique<folly::AsyncSocketException>(
@@ -144,10 +143,7 @@ folly::AsyncSocket::ReadResult AsyncKTLSSocket::handleNonApplicationData(
 }
 
 folly::AsyncSocket::ReadResult AsyncKTLSSocket::processAlert(
-    folly::ByteRange payload) {
-  auto payloadBuf =
-      folly::IOBuf::wrapBufferAsValue(payload.data(), payload.size());
-
+    folly::IOBufQueue payload) {
   fizz::Alert alert;
   try {
     // Unlike handshake data, we do not buffer alerts that could not be fully
@@ -163,7 +159,7 @@ folly::AsyncSocket::ReadResult AsyncKTLSSocket::processAlert(
     //
     // Since all alerts signal the abort of a TLS session, we do not need
     // to worry about additional application data.
-    folly::io::Cursor cursor(&payloadBuf);
+    folly::io::Cursor cursor(payload.front());
     alert = fizz::decode<fizz::Alert>(cursor);
   } catch (std::exception&) {
     return ReadResult(
@@ -190,7 +186,7 @@ folly::AsyncSocket::ReadResult AsyncKTLSSocket::processAlert(
 }
 
 folly::AsyncSocket::ReadResult AsyncKTLSSocket::processHandshakeData(
-    folly::ByteRange payload) {
+    folly::IOBufQueue payload) {
   // [note.handshake_fits_in_record_assumption]
   //
   // It is extremely likely (unless the peer is malicious, or the user
@@ -199,35 +195,29 @@ folly::AsyncSocket::ReadResult AsyncKTLSSocket::processHandshakeData(
   // peer can fit in one record (and consequently, can be handled directly
   // by this call).
   //
-  // We try to parse the handshake message directly off of `payloadBuf`
+  // We try to parse the handshake message directly off of `payload`
   // first, before we fall back to copying the contents of the buffer into
   // an internal IOBufQueue.
-  auto payloadBuf =
-      folly::IOBuf::wrapBufferAsValue(payload.data(), payload.size());
-
   VLOG(10) << "AsyncKTLSSocket::processHandshakeData()";
   folly::Optional<fizz::Param> handshakeMessage;
 
   // TODO: This can probably be simplified
   try {
     if (FOLLY_LIKELY(unparsedHandshakeData_ == nullptr)) {
-      // TODO: Refactor fizz::ReadRecordLayer decoding logic to work on IOBuf
-      // so we can avoid constructing an iobufqueue with a single element (
-      // this forces a heap allocation...)
-      folly::IOBufQueue tmp{folly::IOBufQueue::cacheChainLength()};
-      tmp.append(payloadBuf);
-      handshakeMessage = fizz::ReadRecordLayer::decodeHandshakeMessage(tmp);
+      handshakeMessage = fizz::ReadRecordLayer::decodeHandshakeMessage(payload);
       if (!handshakeMessage) {
         unparsedHandshakeData_ = std::make_unique<folly::IOBufQueue>(
             folly::IOBufQueue::cacheChainLength());
-        payloadBuf.makeManaged();
-        unparsedHandshakeData_->append(payloadBuf.clone());
+        auto buf = payload.move();
+        buf->makeManaged();
+        unparsedHandshakeData_->append(buf->clone());
         return ReadResult(READ_BLOCKING);
       }
     } else {
       // We already have data saved from last time
-      payloadBuf.makeManaged();
-      unparsedHandshakeData_->append(payloadBuf.clone());
+      auto buf = payload.move();
+      buf->makeManaged();
+      unparsedHandshakeData_->append(buf->clone());
       handshakeMessage = fizz::ReadRecordLayer::decodeHandshakeMessage(
           *unparsedHandshakeData_);
       if (!handshakeMessage) {
