@@ -153,6 +153,16 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
      * to satisfy this read.
      */
     size_t readBufferMinReadSize{1460};
+    /**
+     * When set, `zeroCopyMemStore` points to an instance of a
+     * `ZeroCopyMemStore` that outlives all `AsyncFizzBase` instances.
+     *
+     * When set, Fizz will attempt to perform TCP zero copy receives. This
+     * requires appropriate kernel and hardware support. With appropriate
+     * support, encrypted data received by the NIC will be handed directly to
+     * Fizz for decryption (normally, Fizz will copy data from the kernel).
+     */
+    ZeroCopyMemStore* zeroCopyMemStore{nullptr};
   };
 
   explicit AsyncFizzBase(
@@ -204,6 +214,11 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
    */
   virtual folly::Optional<CipherSuite> getCipher() const = 0;
 
+  /**
+   * Get the NamedGroup negotiated in this transport.
+   */
+  virtual folly::Optional<NamedGroup> getGroup() const = 0;
+
   /**
    * Get the supported signature schemes in this transport.
    */
@@ -315,10 +330,22 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
 
   /*
    * Used to shut down the tls session, without shutting down the underlying
-   * transport. Note you will still need to set setCloseTransportOnCloseNotify.
+   * transport.
    */
   virtual void tlsShutdown() = 0;
 
+  /*
+   * Redeclared from AsyncTransport. Attempts to perform a full TLS shutdown,
+   * then shuts down the underlying transport.
+   */
+  virtual void shutdownWrite() override = 0;
+
+  /*
+   * Redeclared from AsyncTransport. Attempts to perform a full TLS shutdown,
+   * then shuts down the underlying transport.
+   */
+  virtual void shutdownWriteNow() override = 0;
+
   /*
    * Sets whether or not to force in-place decryption of records. This is
    * usually safe to do, as long as the application can handle chained IOBufs
@@ -498,6 +525,14 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
       std::unique_ptr<folly::IOBuf> data) noexcept override;
   void readEOF() noexcept override;
   void readErr(const folly::AsyncSocketException& ex) noexcept override;
+  /* ZC RX related*/
+  ZeroCopyMemStore* readZeroCopyEnabled() noexcept override;
+  void getZeroCopyFallbackBuffer(
+      void** /*bufReturn*/,
+      size_t* /*lenReturn*/) noexcept override;
+  void readZeroCopyDataAvailable(
+      std::unique_ptr<folly::IOBuf>&& /*zeroCopyData*/,
+      size_t /*additionalBytes*/) noexcept override;
 
   /**
    * WriteCallback implementation, for use with handshake messages.
@@ -511,6 +546,9 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
 
   void handshakeTimeoutExpired() noexcept;
 
+  void
+  getReadBuffer(folly::IOBufQueue& buf, void** bufReturn, size_t* lenReturn);
+
   ReadCallback* readCallback_{nullptr};
   std::unique_ptr<folly::IOBuf> appDataBuf_;
 
@@ -528,5 +566,8 @@ class AsyncFizzBase : public folly::WriteChainAsyncTransportWrapper<
 
   TransportOptions transportOptions_;
   std::unique_ptr<FizzMsgHdr> msgHdr_;
+
+  folly::IOBufQueue zeroCopyFallbackReadBuf_{
+      folly::IOBufQueue::cacheChainLength()};
 };
 } // namespace fizz
