@@ -9,6 +9,7 @@
 #pragma once
 
 #include <fizz/protocol/CertificateVerifier.h>
+#include <folly/ssl/OpenSSLPtrTypes.h>
 
 namespace fizz {
 
@@ -45,6 +46,14 @@ class DefaultCertificateVerifier : public CertificateVerifier {
       const std::vector<std::shared_ptr<const fizz::PeerCert>>& certs)
       const override;
 
+  /**
+   * Return a std::unique_ptr to X509_STORE_CTX which is used for verifying
+   * the certificate chain. Additional verification result can be extracted
+   * from the returned store context.
+   */
+  folly::ssl::X509StoreCtxUniquePtr verifyWithX509StoreCtx(
+      const std::vector<std::shared_ptr<const fizz::PeerCert>>& certs) const;
+
   void setCustomVerifyCallback(X509VerifyCallback cb) {
     customVerifyCallback_ = cb;
   }
