@@ -19,7 +19,9 @@ namespace ech {
 
 struct SupportedECHConfig {
   ECHConfig config;
-  ECHCipherSuite cipherSuite;
+  uint8_t configId;
+  uint16_t maxLen;
+  HpkeSymmetricCipherSuite cipherSuite;
 };
 
 // Used to indicate to the Decrypter that extension expansion failed (which is
@@ -40,56 +42,77 @@ hpke::SetupResult constructHpkeSetupResult(
     const SupportedECHConfig& supportedConfig);
 
 std::unique_ptr<folly::IOBuf> makeClientHelloAad(
-    ECHCipherSuite cipherSuite,
-    const std::unique_ptr<folly::IOBuf>& configId,
+    HpkeSymmetricCipherSuite cipherSuite,
+    const uint8_t configId,
     const std::unique_ptr<folly::IOBuf>& enc,
     const std::unique_ptr<folly::IOBuf>& clientHello);
 
+folly::Optional<ClientPresharedKey> generateGreasePSK(
+    const ClientHello& chloInner,
+    const Factory* factory);
+
+ClientPresharedKey generateGreasePSKForHRR(
+    const ClientPresharedKey& previousPsk,
+    const Factory* factory);
+
 ServerHello makeDummyServerHello(const ServerHello& shlo);
 
+HelloRetryRequest makeDummyHRR(const HelloRetryRequest& hrr);
+
 bool checkECHAccepted(
     const ServerHello& shlo,
     std::unique_ptr<HandshakeContext> context,
-    std::unique_ptr<KeyScheduler>& scheduler);
+    std::unique_ptr<KeyScheduler> scheduler);
+
+bool checkECHAccepted(
+    const HelloRetryRequest& shlo,
+    std::unique_ptr<HandshakeContext> context,
+    std::unique_ptr<KeyScheduler> scheduler);
 
 void setAcceptConfirmation(
     ServerHello& shlo,
     std::unique_ptr<HandshakeContext> context,
-    std::unique_ptr<KeyScheduler>& scheduler);
+    std::unique_ptr<KeyScheduler> scheduler);
 
-ClientECH encryptClientHelloHRR(
+void setAcceptConfirmation(
+    HelloRetryRequest& hrr,
+    std::unique_ptr<HandshakeContext> context,
+    std::unique_ptr<KeyScheduler> scheduler);
+
+size_t
+calculateECHPadding(const ClientHello& chlo, size_t encodedSize, size_t maxLen);
+
+OuterECHClientHello encryptClientHelloHRR(
     const SupportedECHConfig& supportedConfig,
     const ClientHello& clientHelloInner,
     const ClientHello& clientHelloOuter,
-    hpke::SetupResult& setupResult);
+    hpke::SetupResult& setupResult,
+    const folly::Optional<ClientPresharedKey>& greasePsk);
 
-ClientECH encryptClientHello(
+OuterECHClientHello encryptClientHello(
     const SupportedECHConfig& supportedConfig,
     const ClientHello& clientHelloInner,
     const ClientHello& clientHelloOuter,
-    hpke::SetupResult& setupResult);
+    hpke::SetupResult& setupResult,
+    const folly::Optional<ClientPresharedKey>& greasePsk);
 
 ClientHello decryptECHWithContext(
     const ClientHello& clientHelloOuter,
     const ECHConfig& echConfig,
-    ECHCipherSuite& cipherSuite,
+    HpkeSymmetricCipherSuite& cipherSuite,
     std::unique_ptr<folly::IOBuf> encapsulatedKey,
-    std::unique_ptr<folly::IOBuf> configId,
+    uint8_t configId,
     std::unique_ptr<folly::IOBuf> encryptedCh,
     ECHVersion version,
     std::unique_ptr<hpke::HpkeContext>& context);
 
 std::unique_ptr<hpke::HpkeContext> setupDecryptionContext(
     const ECHConfig& echConfig,
-    ECHCipherSuite cipherSuite,
+    HpkeSymmetricCipherSuite cipherSuite,
     const std::unique_ptr<folly::IOBuf>& encapsulatedKey,
     std::unique_ptr<KeyExchange> kex,
     uint64_t seqNum);
 
-std::unique_ptr<folly::IOBuf> constructConfigId(
-    hpke::KDFId kdfId,
-    ECHConfig echConfig);
-
 std::unique_ptr<folly::IOBuf> getRecordDigest(
     const ECHConfig& echConfig,
     hpke::KDFId id);
