@@ -19,6 +19,7 @@
 #include <fizz/protocol/KeyScheduler.h>
 #include <fizz/protocol/OpenSSLFactory.h>
 #include <fizz/protocol/Types.h>
+#include <fizz/protocol/ech/Decrypter.h>
 #include <fizz/record/test/Mocks.h>
 
 #include <folly/io/async/test/MockAsyncTransport.h>
@@ -378,6 +379,7 @@ class MockAsyncFizzBase : public AsyncFizzBase {
   }
 
   MOCK_METHOD(folly::Optional<CipherSuite>, getCipher, (), (const));
+  MOCK_METHOD(folly::Optional<NamedGroup>, getGroup, (), (const));
   MOCK_METHOD(
       std::vector<SignatureScheme>,
       getSupportedSigSchemes,
@@ -398,6 +400,8 @@ class MockAsyncFizzBase : public AsyncFizzBase {
 
   MOCK_METHOD(folly::Optional<Random>, getClientRandom, (), (const));
   MOCK_METHOD(void, tlsShutdown, ());
+  MOCK_METHOD(void, shutdownWrite, (), (override));
+  MOCK_METHOD(void, shutdownWriteNow, (), (override));
   MOCK_METHOD(void, initiateKeyUpdate, (KeyUpdateRequest), (override));
 
   MOCK_METHOD(
@@ -422,5 +426,41 @@ class MockAsyncFizzBase : public AsyncFizzBase {
   MOCK_METHOD(void, resumeEvents, ());
 };
 
+class MockECHDecrypter : public ech::Decrypter {
+ public:
+  MOCK_METHOD(
+      folly::Optional<ech::DecrypterResult>,
+      decryptClientHello,
+      (const ClientHello& chlo));
+
+  MOCK_METHOD(
+      ClientHello,
+      _decryptClientHelloHRR_Stateful,
+      (const ClientHello& chlo, std::unique_ptr<hpke::HpkeContext>& context));
+
+  ClientHello decryptClientHelloHRR(
+      const ClientHello& chlo,
+      std::unique_ptr<hpke::HpkeContext>& context) override {
+    return _decryptClientHelloHRR_Stateful(chlo, context);
+  }
+
+  MOCK_METHOD(
+      ClientHello,
+      _decryptClientHelloHRR_Stateless,
+      (const ClientHello& chlo,
+       const std::unique_ptr<folly::IOBuf>& encapsulatedKey));
+
+  ClientHello decryptClientHelloHRR(
+      const ClientHello& chlo,
+      const std::unique_ptr<folly::IOBuf>& encapsulatedKey) override {
+    return _decryptClientHelloHRR_Stateless(chlo, encapsulatedKey);
+  }
+
+  MOCK_METHOD(
+      std::vector<ech::ECHConfig>,
+      getRetryConfigs,
+      (),
+      (const, override));
+};
 } // namespace test
 } // namespace fizz
