@@ -101,8 +101,6 @@ std::string toString(ExtensionType extType) {
       return "encrypted_client_hello";
     case ExtensionType::ech_nonce:
       return "ech_nonce";
-    case ExtensionType::ech_is_inner:
-      return "ech_is_inner";
     case ExtensionType::ech_outer_extensions:
       return "ech_outer_extensions";
   }
@@ -247,6 +245,20 @@ std::string toString(NamedGroup group) {
       return "x25519";
     case NamedGroup::secp521r1_x25519:
       return "secp521r1_x25519";
+    case NamedGroup::secp384r1_bikel3:
+      return "secp384r1_bikel3";
+    case NamedGroup::cecpq2:
+      return "cecpq2";
+    case NamedGroup::x25519_kyber512:
+      return "x25519_kyber512";
+    case NamedGroup::secp256r1_kyber512:
+      return "secp256r1_kyber512";
+    case NamedGroup::kyber512:
+      return "kyber512";
+    case NamedGroup::x25519_kyber768:
+      return "x25519_kyber768";
+    case NamedGroup::secp384r1_kyber768:
+      return "secp384r1_kyber768";
   }
   return enumToHex(group);
 }
