@@ -114,6 +114,26 @@ void AsyncFizzServerT<SM>::tlsShutdown() {
   }
 }
 
+template <typename SM>
+void AsyncFizzServerT<SM>::shutdownWrite() {
+  DelayedDestruction::DestructorGuard dg(this);
+  // Attempt to send a close_notify, then perform a half-shutdown of the write
+  // side of the underlying socket. If previously submitted writes, and the
+  // close_notify itself, are queued up in the Fizz layer, they won't make it
+  // out before the socket shuts down.
+  tlsShutdown();
+  transport_->shutdownWrite();
+}
+
+template <typename SM>
+void AsyncFizzServerT<SM>::shutdownWriteNow() {
+  DelayedDestruction::DestructorGuard dg(this);
+  // Similar to shutdownWrite(), attempt to write out the close_notify, but it
+  // might not make it out to the transport layer before the socket is closed.
+  tlsShutdown();
+  transport_->shutdownWriteNow();
+}
+
 template <typename SM>
 void AsyncFizzServerT<SM>::closeWithReset() {
   DelayedDestruction::DestructorGuard dg(this);
@@ -150,6 +170,11 @@ folly::Optional<CipherSuite> AsyncFizzServerT<SM>::getCipher() const {
   return getState().cipher();
 }
 
+template <typename SM>
+folly::Optional<NamedGroup> AsyncFizzServerT<SM>::getGroup() const {
+  return getState().group();
+}
+
 template <typename SM>
 std::vector<SignatureScheme> AsyncFizzServerT<SM>::getSupportedSigSchemes()
     const {
@@ -343,16 +368,13 @@ void AsyncFizzServerT<SM>::ActionMoveVisitor::operator()(
 template <typename SM>
 void AsyncFizzServerT<SM>::ActionMoveVisitor::operator()(
     SecretAvailable& secret) {
-#if 0
-  FOLLY_SDT(
-      fizz,
-      fizz_secret_available,
+  fizz_probe_secret_available(
       secret.secret.secret.size(),
       secret.secret.secret.data(),
       KeyLogWriter::secretToNSSLabel(secret.secret.type)
           .value_or(std::numeric_limits<KeyLogWriter::Label>::max()),
       server_.getClientRandom()->data());
-#endif
+
   server_.secretAvailable(secret.secret);
 }
 
