@@ -13,8 +13,7 @@
 #include <fizz/server/FizzServer.h>
 #include <fizz/server/FizzServerContext.h>
 #include <fizz/server/ServerProtocol.h>
-#include <fizz/util/KeyLogWriter.h>
-#include <folly/tracing/StaticTracepoint.h>
+#include <fizz/util/Tracing.h>
 
 namespace fizz {
 namespace server {
@@ -61,6 +60,8 @@ class AsyncFizzServerT : public AsyncFizzBase {
   std::string getApplicationProtocol() const noexcept override;
 
   void tlsShutdown() override;
+  void shutdownWrite() override;
+  void shutdownWriteNow() override;
   void close() override;
   void closeWithReset() override;
   void closeNow() override;
@@ -80,6 +81,8 @@ class AsyncFizzServerT : public AsyncFizzBase {
 
   folly::Optional<CipherSuite> getCipher() const override;
 
+  folly::Optional<NamedGroup> getGroup() const override;
+
   std::vector<SignatureScheme> getSupportedSigSchemes() const override;
 
   Buf getExportedKeyingMaterial(
