@@ -66,6 +66,7 @@ class AsyncFizzBaseTest : public testing::Test, public AsyncFizzBase {
   }
 
   MOCK_METHOD(folly::Optional<CipherSuite>, getCipher, (), (const));
+  MOCK_METHOD(folly::Optional<NamedGroup>, getGroup, (), (const));
   MOCK_METHOD(
       std::vector<SignatureScheme>,
       getSupportedSigSchemes,
@@ -99,6 +100,8 @@ class AsyncFizzBaseTest : public testing::Test, public AsyncFizzBase {
 
   MOCK_METHOD(folly::Optional<Random>, getClientRandom, (), (const));
   MOCK_METHOD(void, tlsShutdown, ());
+  MOCK_METHOD(void, shutdownWrite, ());
+  MOCK_METHOD(void, shutdownWriteNow, ());
   MOCK_METHOD(void, initiateKeyUpdate, (KeyUpdateRequest), (override));
 
  protected:
