@@ -23,6 +23,7 @@
 #include <fizz/server/SlidingBloomReplayCache.h>
 #include <fizz/server/TicketTypes.h>
 #include <fizz/tool/FizzCommandCommon.h>
+#include <fizz/util/FizzUtil.h>
 #include <fizz/util/KeyLogWriter.h>
 #include <fizz/util/Parse.h>
 #ifdef FIZZ_TOOL_ENABLE_OQS
@@ -31,7 +32,6 @@
 #include <folly/Format.h>
 #include <folly/io/async/AsyncSSLSocket.h>
 #include <folly/io/async/AsyncServerSocket.h>
-
 #include <fstream>
 #include <string>
 #include <vector>
@@ -569,59 +569,6 @@ class PrivateKeyMatcher {
   std::unordered_map<std::string, folly::ssl::EvpPkeyUniquePtr> privKeys_;
 };
 
-std::unique_ptr<KeyExchange> createKeyExchange(
-    hpke::KEMId kemId,
-    std::string echPrivateKeyFile) {
-  auto getPrivateKey = [&echPrivateKeyFile]() {
-    std::string keyData;
-    folly::ssl::EvpPkeyUniquePtr privKey;
-    if (readFile(echPrivateKeyFile.c_str(), keyData)) {
-      privKey = CertUtils::readPrivateKeyFromBuffer(keyData);
-    }
-    return privKey;
-  };
-
-  switch (kemId) {
-    case hpke::KEMId::secp256r1: {
-      auto kex = std::make_unique<OpenSSLECKeyExchange<P256>>();
-      kex->setPrivateKey(getPrivateKey());
-      return kex;
-    }
-    case hpke::KEMId::secp384r1: {
-      auto kex = std::make_unique<OpenSSLECKeyExchange<P384>>();
-      kex->setPrivateKey(getPrivateKey());
-      return kex;
-    }
-    case hpke::KEMId::secp521r1: {
-      auto kex = std::make_unique<OpenSSLECKeyExchange<P521>>();
-      kex->setPrivateKey(getPrivateKey());
-      return kex;
-    }
-    case hpke::KEMId::x25519: {
-      auto kex = std::make_unique<X25519KeyExchange>();
-      std::string keyData;
-      std::ifstream infile(echPrivateKeyFile);
-
-      // Assume the first line is the private key in hex, the second line is the
-      // public key in hex.
-      std::string privKeyStr, pubKeyStr;
-      infile >> privKeyStr;
-      infile >> pubKeyStr;
-
-      kex->setKeyPair(
-          folly::IOBuf::copyBuffer(folly::unhexlify(privKeyStr)),
-          folly::IOBuf::copyBuffer(folly::unhexlify(pubKeyStr)));
-
-      return kex;
-    }
-    default: {
-      // We don't support other key exchanges right now.
-      break;
-    }
-  }
-  return nullptr;
-}
-
 std::shared_ptr<ech::Decrypter> setupDecrypterFromInputs(
     std::string echConfigsFile,
     std::string echPrivateKeyFile) {
@@ -649,7 +596,8 @@ std::shared_ptr<ech::Decrypter> setupDecrypterFromInputs(
       getKEMId((*echConfigsJson)["echconfigs"][0]["kem_id"].asString());
 
   // Create a key exchange and set the private key
-  auto kexWithPrivateKey = createKeyExchange(kemId, echPrivateKeyFile);
+  auto kexWithPrivateKey =
+      fizz::FizzUtil::createKeyExchange(kemId, echPrivateKeyFile);
   if (!kexWithPrivateKey) {
     LOG(ERROR)
         << "Unable to create a key exchange and set a private key for it.";
